#!/usr/bin/env python


import os
import glob
import fnmatch
from gimpfu import *



def og_paintedworld_normals(image, selectedLayer, backgroundThreshold, oilMaskRadius, oilExponent, oilIterations, oilUseIntensity, oilBlurRadius, realLayerStrength, paintLayerStrength, specAmount):
	try:
		# Delete all layers with mipmap in the name
		for layer in image.layers: 
			if "mipmap" in layer.name:
				image.remove_layer(layer)
		
		# Transfer alpha to layer mask, then delete it. Then, replace any white with (0.5 0.5 1.0)
		for layer in image.layers: 
			mask = layer.create_mask(ADD_ALPHA_TRANSFER_MASK)
			layer.add_mask(mask)
			layer.remove_mask(MASK_DISCARD)
			pdb['plug-in-exchange'](image, layer, 255, 255, 255, 128, 128, 255, backgroundThreshold, backgroundThreshold, backgroundThreshold)
				
		# Detect layers to work with
		realLayer = image.layers[0]
		paintLayer = image.layers[1]

		# Create a layer that is all white
		whiteLayer = gimp.Layer(image, "white mask", realLayer.width, realLayer.height, image.base_type, 100, NORMAL_MODE)
		gimp.set_background(255, 255, 255)
		whiteLayer.fill(BACKGROUND_FILL)
		image.add_layer(whiteLayer)

		# Apply oil filter to first layer, oilIterations times
		for iterations in range(int(oilIterations)):
			pdb['plug-in-oilify-enhanced'](image, realLayer, oilUseIntensity, oilMaskRadius, whiteLayer, oilExponent, whiteLayer) 
			
		# Delete white layer
		image.remove_layer(whiteLayer)
		
		# Gaussian blur (radius 5)
		pdb.plug_in_gauss_rle(image, realLayer, oilBlurRadius, 1, 1)

		# Set image to 2nd layer size
		width = paintLayer.width
		height = paintLayer.height
		image.resize(width, height, 0, 0)

		# Scale first layer to image size
		realLayer.scale(width, height, 1)
		
		#make sure both layers are aligned correctly
		realLayer.set_offsets(0, 0)
		paintLayer.set_offsets(0, 0)
		
		# Add a "flat normal" layer and Grain Extract it from the first layer
		flatLayer = gimp.Layer(image, "flat normal", realLayer.width, realLayer.height, image.base_type, 100, GRAIN_EXTRACT_MODE)
		gimp.set_background(128, 128, 255)
		flatLayer.fill(BACKGROUND_FILL)
		image.add_layer(flatLayer, 0)
		realLayerDetails = image.merge_down(flatLayer, CLIP_TO_BOTTOM_LAYER)
		
		# Set first layer to grain merge
		realLayerDetails.mode = GRAIN_MERGE_MODE
		
		# Set first layer alpha to inRealisticStrength
		realLayerDetails.opacity = realLayerStrength
		
		# Blend style layer with a flat normal to weaken it
		flatLayer2 = gimp.Layer(image, "flat normal 2", paintLayer.width, paintLayer.height, image.base_type, 100, NORMAL_MODE)
		gimp.set_background(128, 128, 255)
		flatLayer2.fill(BACKGROUND_FILL)
		image.add_layer(flatLayer2, 2)
		paintLayer.opacity = paintLayerStrength
		paintLayerWeakened = image.merge_down(paintLayer, CLIP_TO_BOTTOM_LAYER)
		
		# Merge down
		mergedLayer = image.merge_visible_layers(0)
		
		# Add layer mask
		mask = mergedLayer.create_mask(ADD_ALPHA_TRANSFER_MASK)
		mergedLayer.add_mask(mask)
		gimp.set_background(specAmount, specAmount, specAmount)
		mask.fill(BACKGROUND_FILL)
		mergedLayer.remove_mask(MASK_APPLY)
	
	except Exception as err:
		gimp.message("Unexpected error: " + str(err))
		

def find_files(directory, pattern):
    for root, dirs, files in os.walk(directory):
        for basename in files:
            if fnmatch.fnmatch(basename, pattern):
                filename = os.path.join(root, basename)
                yield filename
				
				
def remove_prefix(s, prefix):
    return s[len(prefix):] if s.startswith(prefix) else s


def og_batch_paintedworld_normals(img, layer, importRealFolder, importStyleFolder, exportFolder, exportDDS, backgroundThreshold, oilMaskRadius, oilExponent, oilIterations, oilUseIntensity, oilBlurRadius, realLayerStrength, paintLayerStrength, specAmount):
	#Fix backslashes
	importRealFolder = importRealFolder.replace("\\", "/")
	importStyleFolder = importStyleFolder.replace("\\", "/")
	exportFolder = exportFolder.replace("\\", "/")
	
	try:
		for inputStylePath in find_files(importStyleFolder, '*.*'):
			
			#Load the "style" normal
			inputStylePath = inputStylePath.replace("\\", "/")
			print(inputStylePath)
			styleImage = None
			styleImage = pdb.gimp_file_load(inputStylePath, inputStylePath)
			if (styleImage == None):
				return
			print(styleImage)
			# pdb.gimp_display_new(styleImage) # For testing only! Remove in final version.
			
			#Load the "real normal"
			fileName = inputStylePath.rsplit('.', 1)[0] # remove extension
			fileName = remove_prefix(fileName, importStyleFolder) # remove folder path
			inputRealPathPartial = importRealFolder + fileName + ".*" # change folder path, unknown extension
			inputRealPathPartial = inputRealPathPartial.replace("\\", "/")
			inputRealPath = glob.glob(inputRealPathPartial)[0] # search for a matching file
			inputRealPath = inputRealPath.replace("\\", "/")
			print(inputRealPath)
			realLayer = pdb.gimp_file_load_layer(styleImage, inputRealPath) # load as layer
			if (realLayer == None):
				return
			print(realLayer)
			styleImage.add_layer(realLayer, 0) # add this layer to top of image stack
			
			# Process the layers
			og_paintedworld_normals(styleImage, realLayer, backgroundThreshold, oilMaskRadius, oilExponent, oilIterations, oilUseIntensity, oilBlurRadius, realLayerStrength, paintLayerStrength, specAmount)
			
			# Export result
			resultLayer = styleImage.layers[0]
			outputPath = exportFolder + fileName
			outputDir = outputPath.rsplit("/", 1)[0]
			if not os.path.exists(outputDir):
				os.makedirs(outputDir)
			print(outputPath)
			if (exportDDS):
				outputPath += ".dds"
				pdb.file_dds_save(styleImage, resultLayer, outputPath, outputPath, 3, 1, 0, 0, -1, 0, 0, 0, 0, 2.2, 0, 0, 0.5)
			else:
				outputPath += ".png"
				pdb.file_png_save_defaults(styleImage, resultLayer, outputPath, outputPath)
			
			# Delete the image
			pdb.gimp_image_delete(styleImage)
			
	except Exception as err:
		gimp.message("Unexpected error: " + str(err))
		
		
register(
	"og_paintedworld_normals",
	"PaintedWorld normals",
	"Uses a vanilla, realistic normal map and a map extracted from the oil texture to produce a normal for the Painted World",
	"opusGlass",
	"open source (MIT license)",
	"2020",
	"<Image>/opusGlass/PaintedWorld normals",
	"*",
	[
		(PF_ADJUSTMENT, "WhiteBackgroundThreshold", "White background threshold: ", 10, (0, 50, 1)),
		(PF_ADJUSTMENT, "OilifyMaskRadius", "Oilify mask radius: ", 14, (1, 32, 1)),
		(PF_ADJUSTMENT, "OilifyExponent", "Oilify exponent: ", 2, (2, 20, 1)),
		(PF_ADJUSTMENT, "OilifyNumberOfIterations", "Oilify number of iterations: ", 8, (0, 50, 1)),
		(PF_TOGGLE, "OilifyIntensityMode", "Oilify use intensity: ", 1),
		(PF_ADJUSTMENT, "OilBlurRadius", "Oil blur radius: ", 5.0, (0, 100, 0.5)),
		(PF_ADJUSTMENT, "OilifiedRealisticLayerStrength", "Oilified realistic layer strength: ", 100.0, (0, 100, 1.0)),
		(PF_ADJUSTMENT, "PaintLayerStrength", "Painted (style) layer strength: ", 50.0, (0, 100, 1.0)),
		(PF_ADJUSTMENT, "SpecularAmount", "Specular amount: ", 0.42, (0, 1, 0.01))
	],
	[],
	og_paintedworld_normals)


register(
    "og_batch_paintedworld_normals",
    "Batch PaintedWorld normals",
    "Uses a vanilla, realistic normal map and a map extracted from the oil texture to produce a normal for the Painted World",
	"opusGlass",
	"open source (MIT license)",
	"2020",
	"<Image>/opusGlass/Batch PaintedWorld normals",
    "*",
    [
        (PF_DIRNAME, "importRealFolder", "Import Realistic/Vanilla normal directory:", "D:/Drive/Skyrim/modding/PaintedWorld/gimp/normals/vanilla"),
        (PF_DIRNAME, "importStyleFolder", "Import Style normal directory:", "D:/Drive/Skyrim/modding/PaintedWorld/gimp/normals/minerman_sorted"),
        (PF_DIRNAME, "exportFolder", "Export directory", "D:/Drive/Skyrim/modding/PaintedWorld/gimp/normals/export"),
		(PF_TOGGLE, "exportFormat", "Export as compressed DDS? (else PNG) : ", 1),
		(PF_ADJUSTMENT, "WhiteBackgroundThreshold", "White background threshold: ", 10, (0, 50, 1)),
		(PF_ADJUSTMENT, "OilifyMaskRadius", "Oilify mask radius: ", 14, (1, 32, 1)),
		(PF_ADJUSTMENT, "OilifyExponent", "Oilify exponent: ", 2, (2, 20, 1)),
		(PF_ADJUSTMENT, "OilifyNumberOfIterations", "Oilify number of iterations: ", 8, (0, 50, 1)),
		(PF_TOGGLE, "OilifyIntensityMode", "Oilify use intensity: ", 1),
		(PF_ADJUSTMENT, "OilBlurRadius", "Oil blur radius: ", 5.0, (0, 100, 0.5)),
		(PF_ADJUSTMENT, "OilifiedRealisticLayerStrength", "Oilified realistic layer strength: ", 100.0, (0, 100, 1.0)),
		(PF_ADJUSTMENT, "PaintLayerStrength", "Painted (style) layer strength: ", 50.0, (0, 100, 1.0)),
		(PF_ADJUSTMENT, "SpecularAmount", "Specular amount: ", 0.42, (0, 1, 0.01))
    ],
    [],
    og_batch_paintedworld_normals)


main()